package property;

import com.pholser.junit.quickcheck.generator.InRange;
import com.pholser.junit.quickcheck.From;
import com.pholser.junit.quickcheck.Property;
import com.pholser.junit.quickcheck.runner.JUnitQuickcheck;
import org.junit.runner.RunWith;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

@RunWith(JUnitQuickcheck.class)
public class CustomerTest {
    @Property
    public void testFullName(@From(CustomerGenerator.class) final Customer c) {
        final String firstName = c.getFirstName();
        final String lastName = c.getLastName();
        final String fullName = c.getFullName();

        assertEquals(firstName.length() + lastName.length() + 1,
                     fullName.length());
        assertTrue(fullName.startsWith(firstName));
        assertTrue(fullName.endsWith(lastName));
    }

    @Property
    public void testNumPurchases(@From(CustomerGenerator.class) final Customer c,
                                 @InRange(min = "1") final int numPurchases) {
        assertEquals(0, c.getNumPurchases());
        for (int purchaseNum = 0; purchaseNum < numPurchases; purchaseNum++) {
            c.makePurchase();
        }
        assertEquals(numPurchases, c.getNumPurchases());
    }
}
