package property;

import com.pholser.junit.quickcheck.generator.Size;
import com.pholser.junit.quickcheck.From;
import com.pholser.junit.quickcheck.Property;
import com.pholser.junit.quickcheck.runner.JUnitQuickcheck;
import org.junit.runner.RunWith;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertEquals;

@RunWith(JUnitQuickcheck.class)
public class MyListCustomTest {
    @Property
    public void testLength(@From(MyListGenerator.class) @Size(min = 0, max = 3) final MyList<Integer> list) {
        final int len = list.length();
        assertTrue(len >= 0);
        assertTrue(len <= 3);
    }

    @Property
    public void testReverseDoubleReverse(@From(MyListGenerator.class) final MyList<Integer> list) {
        assertEquals(list, list.reverse().reverse());
    }

    @Property
    public void testAppendLength(@From(MyListGenerator.class) @Size(max = 5) final MyList<Integer> l1,
                                 @From(MyListGenerator.class) @Size(max = 5) final MyList<Integer> l2) {
        assertEquals(l1.append(l2).length(),
                     l1.length() + l2.length());
        assertTrue(l1.length() + l2.length() <= 10);
    }

    // returns everything after prefix
    public MyList<Integer> assertPrefixSame(MyList<Integer> prefix, MyList<Integer> base) {
        while (prefix instanceof NonEmptyList &&
               base instanceof NonEmptyList) {
            final NonEmptyList<Integer> prefixCast = (NonEmptyList<Integer>)prefix;
            final NonEmptyList<Integer> baseCast = (NonEmptyList<Integer>)base;
            assertEquals(prefixCast.element, baseCast.element);
            prefix = prefixCast.tail;
            base = baseCast.tail;
        }

        assertTrue(prefix instanceof EmptyList);
        return base;
    }

    @Property
    public void testAppendContents(@From(MyListGenerator.class) final MyList<Integer> l1,
                                   @From(MyListGenerator.class) final MyList<Integer> l2) {
        final MyList<Integer> appended = l1.append(l2);
        final MyList<Integer> partWay = assertPrefixSame(l1, appended);
        final MyList<Integer> remainder = assertPrefixSame(l2, partWay);
        assertTrue(remainder instanceof EmptyList);
    }

    @Property
    public void testAppendReverse(@From(MyListGenerator.class) final MyList<Integer> l1,
                                  @From(MyListGenerator.class) final MyList<Integer> l2) {
        // l1: [1,2,3]
        // l2: [4,5,6]
        // l1.reverse(): [3,2,1]
        // l2.reverse(): [6,5,4]
        // l1.append(l2): [1,2,3,4,5,6]
        // l1.append(l2).reverse(): [6,5,4,3,2,1]
        // l2.reverse().append(l1.reverse()): [6,5,4,3,2,1]
        assertEquals(l1.append(l2).reverse(),
                     l2.reverse().append(l1.reverse()));
    }
}
