package property;

import com.pholser.junit.quickcheck.Property;
import com.pholser.junit.quickcheck.runner.JUnitQuickcheck;
import org.junit.runner.RunWith;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.Collections;

// This effectively "cheats" and reuses the fact that junit-quickcheck
// provides generators out of the box for ArrayList
@RunWith(JUnitQuickcheck.class)
public class MyListReuseListTest {
    public static <A> MyList<A> convertList(final ArrayList<A> input) {
        MyList<A> retval = new EmptyList<A>();
        for (int x = input.size() - 1; x >= 0; x--) {
            retval = new NonEmptyList<A>(input.get(x), retval);
        }
        return retval;
    }
    
    @Property
    public void testLength(final ArrayList<Integer> input) {
        assertEquals(input.size(), convertList(input).length());
    }

    @Property
    public void testReverse(final ArrayList<Integer> input) {
        final MyList<Integer> received = convertList(input).reverse();
        Collections.reverse(input);
        assertEquals(convertList(input), received);
    }

    @Property
    public void testAppend(final ArrayList<Integer> first,
                           final ArrayList<Integer> second) {
        final MyList<Integer> received =
            convertList(first).append(convertList(second));
        first.addAll(second);
        assertEquals(convertList(first), received);
    }
}
